/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   When you run this program, watch the console window
   to get information about how your actions are
   interpreted by the Java GUI framework.
*/
public class EventExperiment2
{
   public static void main(String[] args)
   {
      // We have eight objects that are sources of events.
      final JFrame f = new JFrame();  // 1
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("JFrame with panel components.");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);
      f.requestFocus(); // Try commenting this out.

      final JPanel p1 = new JPanel();  // 2
      p1.setBackground(Color.gray);
      p1.setPreferredSize(new Dimension(100, 100));
      f.add(p1);

      final JPanel p2 = new JPanel();  // 3
      p2.setBackground(Color.gray);
      p2.setPreferredSize(new Dimension(100, 100));
      f.add(p2);

      final JPanel p3 = new JPanel();  // 4
      p3.setBackground(Color.gray);
      p3.setPreferredSize(new Dimension(100, 100));
      f.add(p3);

      final JButton b1 = new JButton("JButton 1");  // 5
      f.add(b1);
      final JButton b2 = new JButton("JButton 2");  // 6
      f.add(b2);
      final JButton b3 = new JButton("JB 3");  // 7
      f.add(b3);
      final JButton b4 = new JButton("JB 4"); // 8
      p2.add(b4);

      // Objects that are listeners (handlers) of events.
      f.addMouseListener(new MouseAdapter(){public void mouseEntered(MouseEvent e){System.out.println("JFrame in.");}
                                            public void mouseExited(MouseEvent e){System.out.println("JFrame out.");}
                                            public void mouseClicked(MouseEvent e){System.out.println("JFrame click event.");}});
      f.addKeyListener(new KeyAdapter(){public void keyTyped(KeyEvent e){System.out.println("JFrame key event.");}});
      f.addFocusListener(new FocusListener(){public void focusGained(FocusEvent e){System.out.println("JFrame gained focus.");}
                                             public void focusLost(FocusEvent e){System.out.println("JFrame lost focus.");}});

      p1.addMouseListener(new MouseAdapter(){public void mouseEntered(MouseEvent e){System.out.println("JPanel 1 in.");}
                                             public void mouseExited(MouseEvent e){System.out.println("JPanel 1 out.");}
                                             public void mouseClicked(MouseEvent e){System.out.println("JPanel 1 click event.");}});

      p2.addMouseListener(new MouseAdapter(){public void mouseEntered(MouseEvent e){System.out.println("JPanel 2 in.");}
                                             public void mouseExited(MouseEvent e){System.out.println("JPanel 2 out.");}
                                             public void mouseClicked(MouseEvent e){System.out.println("JPanel 2 click event.");}});

      // Be sure to notice that the space bar on the keyboard triggers an action event (without a key listener)!
      b1.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){System.out.println("JButton 1 action event.");}});

      b2.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){System.out.println("JButton 2 action event.");}});
      b2.addMouseListener(new MouseAdapter(){public void mouseEntered(MouseEvent e){System.out.println("JButton 2 in.");}
                                             public void mouseExited(MouseEvent e){System.out.println("JButton 2 out.");}
                                             public void mouseClicked(MouseEvent e){System.out.println("JButton 2 click event.");}});

      b4.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){System.out.println("JButton 4 action event.");}});
      b4.addMouseListener(new MouseAdapter(){public void mouseEntered(MouseEvent e){System.out.println("JButton 4 in.");}
                                             public void mouseExited(MouseEvent e){System.out.println("JButton 4 out.");}
                                             public void mouseClicked(MouseEvent e){System.out.println("JButton 4 click event.");}});
      b4.addKeyListener(new KeyAdapter(){public void keyTyped(KeyEvent e){System.out.println("JButton 4 key event.");}});
      b4.addFocusListener(new FocusListener(){public void focusGained(FocusEvent e){System.out.println("JButton 4 gained focus.");}
                                              public void focusLost(FocusEvent e){System.out.println("JButton 4 lost focus.");}});
   }
}

/*
NOTE: All the listeners in this program are objects
that are instantiations of an "anonymous inner class".

https://docs.oracle.com/javase/tutorial/java/javaOO/anonymousclasses.html


https://docs.oracle.com/en/java/javase/17/docs/api/java.desktop/java/awt/event/ActionListener.html
https://docs.oracle.com/en/java/javase/17/docs/api/java.desktop/java/awt/event/MouseAdapter.html
https://docs.oracle.com/en/java/javase/17/docs/api/java.desktop/java/awt/event/KeyAdapter.html
https://docs.oracle.com/en/java/javase/17/docs/api/java.desktop/java/awt/event/FocusListener.html


NOTE: Line 79 creates an instance of an anonymous inner subclass
      of an abstract adapter class that implements an interface.
      The anonymous inner subclass overrides three of the empty
      methods from the abstract adapter class.
*/
