/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.
*/
public class EventExperiment2_ver1
{
   public static void main(String[] args)
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI.");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // 1. Object that stores GUI state information.
      final Model model = new Model();

      // 2. GUI component to update when there is an event.
      final View view = new View(model);
      view.setBackground(model.getColorBG());
      view.setForeground(model.getColorFG());
      view.setPreferredSize(new Dimension(200, 100));
      f.add(view);

      // 3. GUI components that are a source of events.
      final JButton b1 = new JButton("Change bg Color");
      f.add(b1);
      final JButton b2 = new JButton("Change fg 1Color");
      f.add(b2);

      // 4. Object that acts as the controller between the model and the view.
      final ActionListener listener1 = new ButtonListener1(model, view);
      final ActionListener listener2 = new ButtonListener2(model, view);

      // Register the listeners with the buttons.
      b1.addActionListener(listener1);
      b2.addActionListener(listener2);
   }
}


// Class that holds the state of the GUI (the model).
class Model
{
   private int colorNumberBG = 0;
   private int colorNumberFG = 0;
   public void incColorNumberBG(){colorNumberBG = (colorNumberBG + 1)%4;}
   public void incColorNumberFG(){colorNumberFG = (colorNumberFG + 1)%4;}
   public Color getColorBG(){
      if (0 == colorNumberBG)
         return Color.red;
      else if (1 == colorNumberBG)
         return Color.green;
      else if (2 == colorNumberBG)
         return Color.blue;
      else
         return Color.yellow;
   }
   public Color getColorFG(){
      if (0 == colorNumberFG)
         return Color.cyan;
      else if (1 == colorNumberFG)
         return Color.magenta;
      else if (2 == colorNumberFG)
         return Color.orange;
      else
         return Color.gray;
   }
}


// Class that displays the GUI (the view).
@SuppressWarnings("serial")
class View extends JPanel
{
   public final Model model;

   View(final Model model){
      this.model = model;
   }
   @Override
   protected void paintComponent(Graphics g)
   {
      final Color cBG = model.getColorBG(); // query the model
      final Color cFG = model.getColorFG();
      this.setBackground(cBG);  // redraw the GUI
      this.setForeground(cFG);
      super.paintComponent(g);
      final Dimension d = getSize();
      g.fillOval(0, 0, d.width, d.height);
   }
}


// Part of the Controller.
class ButtonListener1 implements ActionListener
{
   public final Model model;
   public final View view;

   ButtonListener1(final Model model, final View view){
      this.model = model;
      this.view = view;
   }
   public void actionPerformed(ActionEvent e){
      model.incColorNumberBG();  // update the model
      view.repaint();            // notify the view
   }
}


// Part of the Controller.
class ButtonListener2 implements ActionListener
{
   public final Model model;
   public final View view;

   ButtonListener2(final Model model, final View view){
      this.model = model;
      this.view = view;
   }
   @Override
   public void actionPerformed(ActionEvent e){
      model.incColorNumberFG();  // update the model
      view.repaint();            // notify the view
   }
}
