/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.

   In this version, the model updates the view (as
   described in the Wikipedia article). I don't
   think this is a good idea for this example.

   https://en.wikipedia.org/wiki/Model-view-controller#/media/File:MVC-Process.svg
*/
public class EventExperiment2_ver2
{
   public static void main(String[] args)
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI.");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // 1. GUI component to update when there is an event.
      View view = new View();

      // 2. Object that stores GUI state information (and updates the view).
      final Model model = new Model(view);

      view.setBackground(model.getColorBG());
      view.setForeground(model.getColorFG());
      view.setPreferredSize(new Dimension(200, 100));
      f.add(view);

      // 3. GUI components that are a source of events.
      final JButton b1 = new JButton("Change bg Color");
      f.add(b1);
      final JButton b2 = new JButton("Change fg 1Color");
      f.add(b2);

      // 4. Objects that handle events from the buttons (and are the controllers).
      final ActionListener listener1 = new ButtonListener1(model);
      final ActionListener listener2 = new ButtonListener2(model);

      // Register the listeners with the buttons.
      b1.addActionListener(listener1);
      b2.addActionListener(listener2);
   }
}


// Class that holds the state of the GUI (the model).
class Model
{
   private final View view;

   public Model(final View view){this.view = view;}
   private int colorNumberBG = 0;
   private int colorNumberFG = 0;
   public void incColorNumberBG(){colorNumberBG = (colorNumberBG + 1)%4; update();}
   public void incColorNumberFG(){colorNumberFG = (colorNumberFG + 1)%4; update();}
   public Color getColorBG(){
      if (0 == colorNumberBG)
         return Color.red;
      else if (1 == colorNumberBG)
         return Color.green;
      else if (2 == colorNumberBG)
         return Color.blue;
      else
         return Color.yellow;
   }
   public Color getColorFG(){
      if (0 == colorNumberFG)
         return Color.cyan;
      else if (1 == colorNumberFG)
         return Color.magenta;
      else if (2 == colorNumberFG)
         return Color.orange;
      else
         return Color.gray;
   }
   private void update(){ // update the view
      final Color cBG = getColorBG();
      final Color cFG = getColorFG();
      view.setBackground(cBG);
      view.setForeground(cFG);
      view.repaint();
   }
}


// Class that displays the GUI (the view).
@SuppressWarnings("serial")
class View extends JPanel
{
   @Override
   protected void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      final Dimension d = getSize();
      g.fillOval(0, 0, d.width, d.height);
   }
}


// Part of the Controller.
class ButtonListener1 implements ActionListener
{
   public final Model model;

   ButtonListener1(Model model){
      this.model = model;
   }
   public void actionPerformed(ActionEvent e){
      model.incColorNumberBG(); // update the model
   }
}


// Part of the Controller.
class ButtonListener2 implements ActionListener
{
   public final Model model;

   ButtonListener2(Model model){
      this.model = model;
   }
   @Override
   public void actionPerformed(ActionEvent e){
      model.incColorNumberFG(); // update the model
   }
}
