/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern BUT this
   version combines everything into one single object!
*/
@SuppressWarnings("serial")
public class EventExperiment2_ver3 extends JPanel implements ActionListener
{
   private int colorNumberBG = 0;  // Part of the model.
   private int colorNumberFG = 0;

   /**
      An EventExperiment2_ver3 object is:
         1. the object that stores GUI state information (model),
         2. a GUI component to update when there is an event (view),
         4. the object that handles events from the buttons (listener),
            and the object that acts as the controller between the model and the view.
   */
   public EventExperiment2_ver3()
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI.");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      f.setLocationRelativeTo(null);
      f.setVisible(true);

      // this is a JPanel.
      this.setPreferredSize(new Dimension(200, 100));
      f.add(this);

      // 3. GUI components that are a source of events.
      final JButton b1 = new JButton("Change bg Color");
      f.add(b1);
      final JButton b2 = new JButton("Change fg 1Color");
      f.add(b2);

      // Register this listener with each button.
      b1.addActionListener(this);
      b2.addActionListener(this);
   }

   // This is the model part.
   private void incColorNumberBG()
   {
      colorNumberBG = (colorNumberBG + 1)%4;
   }
   private void incColorNumberFG()
   {
      colorNumberFG = (colorNumberFG + 1)%4;
   }
   private Color getColorBG()
   {
      if (0 == colorNumberBG)
         return Color.red;
      else if (1 == colorNumberBG)
         return Color.green;
      else if (2 == colorNumberBG)
         return Color.blue;
      else
         return Color.yellow;
   }
   private Color getColorFG()
   {
      if (0 == colorNumberFG)
         return Color.cyan;
      else if (1 == colorNumberFG)
         return Color.magenta;
      else if (2 == colorNumberFG)
         return Color.orange;
      else
         return Color.gray;
   }

   // This is the view part, a JPanel.
   @Override
   protected void paintComponent(Graphics g)
   {
      final Color cBG = this.getColorBG(); // query the model
      final Color cFG = this.getColorFG();
      this.setBackground(cBG);  // redraw the GUI
      this.setForeground(cFG);
      super.paintComponent(g);
      final Dimension d = getSize();
      g.fillOval(0, 0, d.width, d.height);
   }

   // This is the controller part.
   @Override
   public void actionPerformed(ActionEvent e)
   {
      if ( e.getActionCommand().equals("Change bg Color") )
      {
         incColorNumberBG(); // update the model
      }
      else
      {
         incColorNumberFG(); // update the model
      }
      this.repaint();  // notify the view
   }


   public static void main(String[] args)
   {
      new EventExperiment2_ver3();
   }
}
