/*

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
   This program is roughly an implementation of the
   Model-view-controller (MVC) design pattern.

   In this version, the Controller class is defined as
   an anonymous local inner class in the program's main
   method.
*/
public class EventExperiment_ver5s
{
   public static void main(String[] args)
   {
      final JFrame f = new JFrame();
      f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      f.setTitle("Interactive GUI v5s");
      f.setSize(600, 600);

      f.setLayout(new FlowLayout());

      // 1. Object that stores GUI state information.
      final Model model = new Model();

      // 2. GUI component to update when there is an event.
      final JPanel view = new View(model);
      view.setPreferredSize(new Dimension(100, 100));
      f.add(view);

      // 4. GUI component that is a source of events.
      final JButton b1 = new JButton("Change Color");
      f.add(b1);

      // 3. Object that handles events from the button (the controller),
      //    defined as an instance of an anonymous inner class.
      b1.addActionListener(
         new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent e){
               model.incColorNumber();  // update the model
               view.repaint();          // notify the view
      }});

      f.setLocationRelativeTo(null);
      f.setVisible(true);
   }
}


// The View in MVC.
@SuppressWarnings("serial")
class View extends JPanel
{
   public final Model state;
   View(final Model state){
      this.state = state;
   }
   @Override
   protected void paintComponent(Graphics g)
   {
      //System.out.println( state.getColorNumber() );
      final Color c = state.getColor();  // query the model
      this.setBackground(c);
      super.paintComponent(g);
   }
}


// The Model in MVC.
class Model
{
   private int colorNumber = 0;
   public void incColorNumber(){colorNumber = (colorNumber+1)%4;}
   public int getColorNumber(){return colorNumber;}
   public Color getColor(){
      if (0 == colorNumber)
         return Color.red;
      else if (1 == colorNumber)
         return Color.green;
      else if (2 == colorNumber)
         return Color.blue;
      else
         return Color.yellow;
   }
}
