/*
    This program allows you to observe various kinds of events
    occur as you interact with a Java graphical user interface.
*/
import java.awt.*;
import java.awt.event.*;
import java.awt.AWTEvent;
import javax.swing.*;
import javax.swing.border.*;

public class WatchEvents
{
   public static void main(String[] args)
   {
      // create an OutputJFrame object for the output from the event listeners
      OutputJFrame outputFrame = new OutputJFrame("View the events here");
      // tell all the event listeners where to send their output
      Listeners.output = outputFrame;
      // display this OutputJFrame
      outputFrame.pack();
      //outputFrame.setLocation(320,50);
      outputFrame.setVisible(true);

      // create the event panel
      JPanel eventPanel = new JPanel();
      // give it a border
      Border panelBorder = new TitledBorder("Event Panel");
      eventPanel.setBorder(panelBorder);

      // create the tabbed pane
      JTabbedPane tabbedPane = new JTabbedPane();

      // Create a CheckBoxJPanel for enabling/disabling each kind of event.
      // Create an array of booleans specifing all the kinds of events.
      // [action, item, text, adjustment, window, component, focus, key, mouse, motion]
      boolean[] eventsEventsEnabled = {true,true,true,true,true,true,true,true,true,true};
      // create the CheckBoxJPanel
      CheckBoxJPanel eventCheckBoxJPanel = new CheckBoxJPanel(eventsEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(eventCheckBoxJPanel, "Events");

      // Now start creating components to put in the event panel
      // along with associated check box panels to put in the
      // tabbedPane and assiciated listeners for the components

      // create a button
      JButton button1 = new JButton("JButton");
      // add it to the event panel
      eventPanel.add(button1);
      // create an array of booleans specifing the events for this button
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] button1EventsEnabled = {true,false,false,false,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the button
      CheckBoxJPanel button1CheckBoxJPanel = new CheckBoxJPanel(button1EventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(button1CheckBoxJPanel, "JButton");
      // create a listener for the button
      Listeners button1Listener = new Listeners(eventCheckBoxJPanel, button1CheckBoxJPanel);
      // register appropriate listeners for the button
      button1.addActionListener(button1Listener);
      button1.addComponentListener(button1Listener);
      button1.addFocusListener(button1Listener);
      button1.addKeyListener(button1Listener);
      button1.addMouseListener(button1Listener);
      button1.addMouseMotionListener(button1Listener);

      // create a combobox
      JComboBox<String> combobox = new JComboBox<>();
      combobox.addItem("Item 1");
      combobox.addItem("Item 2");
      combobox.addItem("Item 3");
      // add it to the event panel
      eventPanel.add(combobox);
      // create an array of booleans specifing the events for this button
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] comboboxEventsEnabled = {true,true,false,false,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the button
      CheckBoxJPanel comboboxCheckBoxJPanel = new CheckBoxJPanel(comboboxEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(comboboxCheckBoxJPanel, "JComboBox");
      // create a listener for the combobox
      Listeners comboboxListener = new Listeners(eventCheckBoxJPanel, comboboxCheckBoxJPanel);
      // register appropriate listeners for the combobox
      combobox.addActionListener(comboboxListener);
      combobox.addItemListener(comboboxListener);
      combobox.addComponentListener(comboboxListener);
      combobox.addFocusListener(comboboxListener);
      combobox.addKeyListener(comboboxListener);
      combobox.addMouseListener(comboboxListener);
      combobox.addMouseMotionListener(comboboxListener);

      // create a check box
      JCheckBox checkBox = new JCheckBox("JCheckBox");
      // add it to the event panel
      eventPanel.add(checkBox);
      // create an array of booleans specifing the events for this check box
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] checkboxEventsEnabled = {true,true,false,false,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the check box
      CheckBoxJPanel checkboxCheckBoxJPanel = new CheckBoxJPanel(checkboxEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(checkboxCheckBoxJPanel, "JCheckBox");
      // create a listener for the button
      Listeners checkboxListener = new Listeners(eventCheckBoxJPanel, checkboxCheckBoxJPanel);
      // register appropriate listeners for the checkbox
      checkBox.addActionListener(checkboxListener);
      checkBox.addItemListener(checkboxListener);
      checkBox.addComponentListener(checkboxListener);
      checkBox.addFocusListener(checkboxListener);
      checkBox.addKeyListener(checkboxListener);
      checkBox.addMouseListener(checkboxListener);
      checkBox.addMouseMotionListener(checkboxListener);

      // create a text field
      TextField textField = new TextField("A text field", 15);
      // add it to the event panel
      eventPanel.add(textField);
      // create an array of booleans specifing the events for this text field
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] textfieldEventsEnabled = {true,false,true,false,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the text field
      CheckBoxJPanel textfieldCheckBoxJPanel = new CheckBoxJPanel(textfieldEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(textfieldCheckBoxJPanel, "TextField");
      // create a listener for the button
      Listeners textfieldListener = new Listeners(eventCheckBoxJPanel, textfieldCheckBoxJPanel);
      // register appropriate listeners for the textfield
      textField.addActionListener(textfieldListener);
      textField.addTextListener(textfieldListener);
      textField.addComponentListener(textfieldListener);
      textField.addFocusListener(textfieldListener);
      textField.addKeyListener(textfieldListener);
      textField.addMouseListener(textfieldListener);
      textField.addMouseMotionListener(textfieldListener);

      // create a horizontal scrollbar
      JScrollBar slider = new JScrollBar(JScrollBar.HORIZONTAL);
      // add it to the event panel
      eventPanel.add(new JLabel("JScrollBar"));
      eventPanel.add(slider);
      // create an array of booleans specifing the events for this scrollbar
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] scrollbarEventsEnabled = {false,false,false,true,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the scrollbar
      CheckBoxJPanel scrollbarCheckBoxJPanel = new CheckBoxJPanel(scrollbarEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(scrollbarCheckBoxJPanel, "JScrollBar");
      // create a listener for the button
      Listeners scrollbarListener = new Listeners(eventCheckBoxJPanel, scrollbarCheckBoxJPanel);
      // register appropriate listeners for the scrollbar
      slider.addAdjustmentListener(scrollbarListener);
      slider.addComponentListener(scrollbarListener);
      slider.addFocusListener(scrollbarListener);
      slider.addMouseListener(scrollbarListener);
      slider.addMouseMotionListener(scrollbarListener);

      // create an array of booleans specifing the events for the eventPanel
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] panelEventsEnabled = {false,false,false,false,false,true,true,true,true,true};
      // create a CheckBoxJPanel for the eventPanel
      CheckBoxJPanel panelCheckBoxJPanel = new CheckBoxJPanel(panelEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(panelCheckBoxJPanel, "JPanel");
      // create a listener for the eventPanel
      Listeners panelListener = new Listeners(eventCheckBoxJPanel, panelCheckBoxJPanel);
      // register appropriate listeners for the eventPanel
      eventPanel.addComponentListener(panelListener);
      eventPanel.addFocusListener(panelListener);
      eventPanel.addKeyListener(panelListener);
      eventPanel.addMouseListener(panelListener);
      eventPanel.addMouseMotionListener(panelListener);

      // create a JFrame object to hold the event panel
      JFrame eventFrame = new JFrame("Create events here");
      // add the event panel to this JFrame
      eventFrame.getContentPane().add(eventPanel);
      // create an array of booleans specifing the events for this JFrame
      // action, item, text, adjustment, window, component, focus, key, mouse, motion
      boolean[] frameEventsEnabled = {false,false,false,false,true,true,true,true,true,true};
      // create a CheckBoxJPanel for the JFrame
      CheckBoxJPanel frameCheckBoxJPanel = new CheckBoxJPanel(frameEventsEnabled);
      // add the checkBoxPanel to the tabbedPane
      tabbedPane.add(frameCheckBoxJPanel, "JFrame");
      // create a listener for the button
      Listeners frameListener = new Listeners(eventCheckBoxJPanel, frameCheckBoxJPanel);
      // register appropriate listeners for the scrollbar
      eventFrame.addWindowListener(frameListener);
      eventFrame.addComponentListener(frameListener);
      eventFrame.addFocusListener(frameListener);
      eventFrame.addKeyListener(frameListener);
      eventFrame.addMouseListener(frameListener);
      eventFrame.addMouseMotionListener(frameListener);
      // display this JFrame
      eventFrame.setSize(400, 400);
      //eventFrame.setLocation(50, 50);
      eventFrame.setVisible(true);

      // create a JFrame object to hold the JTabbedPane
      JFrame tabbedFrame = new JFrame("Choose events here");
      // add the JTabbedPane to the JFrame
      tabbedFrame.getContentPane().add(tabbedPane);
      // display the JFrame
      tabbedFrame.setSize(400, 350);
    //tabbedFrame.setResizable(false);
      //tabbedFrame.setLocation(20, 240);
      tabbedFrame.setVisible(true);
      // listen for the close window event on this JFrame
      // (this uses an anonymous, inner, adapter class)
      tabbedFrame.addWindowListener( new WindowAdapter()
          {public void windowClosing(WindowEvent e){System.exit(0);}});
   }//main
}//WatchEvents


@SuppressWarnings("serial")
class OutputJFrame extends JFrame implements ActionListener
{
   public JTextArea out;
   public String extraline;
   private JPanel buttonPanel;
   private JButton clear;
   private JButton wrap;
   private boolean wrapped = false;
   private JButton separator;

   OutputJFrame(String title)
   {
      // call super class constructor
      super(title);

      // get the frame's content pane
      Container c = getContentPane();

      // set the layout manager for this frame
      c.setLayout( new BorderLayout() );

      // create the text area
      out = new JTextArea(25, 40);
      //out.setEditable(false);
      // put it in a scroll pane
      JScrollPane scrollPane = new JScrollPane(out);
      // add the scroll pane to the frame
      c.add(scrollPane, BorderLayout.CENTER);

      // create a JPanel
      buttonPanel = new JPanel();

      // create a button
      clear = new JButton("Clear");
      // add it to the JPanel
      buttonPanel.add(clear);
      // register an action listener
      clear.addActionListener(this);

      // create a button
      wrap = new JButton("Wrap");
      // add it to the JPanel
      buttonPanel.add(wrap);
      // register an action listener
      wrap.addActionListener(this);

      // create a button
      separator = new JButton("Separator");
      // add it to the JPanel
      buttonPanel.add(separator);
      // register an action listener
      separator.addActionListener(this);

      // add the JPanel to the frame
      c.add(buttonPanel, BorderLayout.SOUTH);
   }//constructor

   // implement ActionListener interface
   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == clear)
      {  // select all the text in the JTextArea
         out.selectAll();
         // move it all to the clipboard
         out.cut();
         // or just delete all the text
         //out.replaceSelection(null)
      }
      else if (e.getSource() == wrap)
      {
         if (!wrapped)
         {
            out.setLineWrap(true);
            wrapped = true;
            extraline = "\n";
         }
         else
         {
            out.setLineWrap(false);
            wrapped = false;
            extraline = "";
         }
      }
      else if (e.getSource() == separator)
      {
         out.append("==================================\n");
      }
   }//actionPerformed
}//OutputJFrame


@SuppressWarnings("serial")
class CheckBoxJPanel extends JPanel implements ItemListener
{
   public boolean printAction = false;
   public boolean printItem = false;
   public boolean printText = false;
   public boolean printAdjustment = false;
   public boolean printWindow = false;
   public boolean printComponent = false;
   public boolean printFocus = false;
   public boolean printKey = false;
   public boolean printMouse = false;
   public boolean printMotion = false;
   // declare check box reference variables
   private JCheckBox action;
   private JCheckBox item;
   private JCheckBox text;
   private JCheckBox adjustment;
   private JCheckBox window;
   private JCheckBox component;
   private JCheckBox focus;
   private JCheckBox key;
   private JCheckBox mouse;
   private JCheckBox motion;

   CheckBoxJPanel(boolean[] enabledEvents)
   {
      // call super class constructor
      super();

      // set the layout manager for this panel
      setLayout( new FlowLayout() );

      // create check box objects
      int i = 0;
      action = new JCheckBox("ActionEvent");
      action.setEnabled(enabledEvents[i++]);
      item = new JCheckBox("ItemEvent");
      item.setEnabled(enabledEvents[i++]);
      text = new JCheckBox("TextEvent");
      text.setEnabled(enabledEvents[i++]);
      adjustment = new JCheckBox("AdjustmentEvent");
      adjustment.setEnabled(enabledEvents[i++]);
      window = new JCheckBox("WindowEvent");
      window.setEnabled(enabledEvents[i++]);
      component = new JCheckBox("ComponentEvent");
      component.setEnabled(enabledEvents[i++]);
      focus = new JCheckBox("FocusEvent");
      focus.setEnabled(enabledEvents[i++]);
      key = new JCheckBox("KeyEvent");
      key.setEnabled(enabledEvents[i++]);
      mouse = new JCheckBox("MouseEvent");
      mouse.setEnabled(enabledEvents[i++]);
      motion = new JCheckBox("MouseEvent (motion)");
      motion.setEnabled(enabledEvents[i++]);

      // add the check boxes to the panel
      JPanel panel1 = new JPanel(new GridLayout(7,1));
      panel1.add(new JLabel("High Level Events"));
      panel1.add( action );
      panel1.add( item );
      panel1.add( text );
      panel1.add( adjustment );
      add(panel1);
      JPanel panel2 = new JPanel(new GridLayout(7,1));
      panel2.add(new JLabel("Low Level Events"));
      panel2.add( window );
      panel2.add( component );
      panel2.add( focus );
      panel2.add( key );
      panel2.add( mouse );
      panel2.add( motion );
      add(panel2);

      // register listeners for the check boxes
      action.addItemListener(this);
      item.addItemListener(this);
      text.addItemListener(this);
      adjustment.addItemListener(this);
      window.addItemListener(this);
      component.addItemListener(this);
      focus.addItemListener(this);
      key.addItemListener(this);
      mouse.addItemListener(this);
      motion.addItemListener(this);
   }//constructor

   // implement the ItemListener interface
   public void itemStateChanged(ItemEvent e)
   {
      if ( action.isSelected() ) printAction = true; else printAction = false;
      if ( item.isSelected() ) printItem = true; else printItem = false;
      if ( text.isSelected() ) printText = true; else printText = false;
      if ( adjustment.isSelected() ) printAdjustment = true; else printAdjustment = false;
      if ( window.isSelected() ) printWindow = true; else printWindow = false;
      if ( component.isSelected() ) printComponent = true; else printComponent = false;
      if ( focus.isSelected() ) printFocus = true; else printFocus = false;
      if ( key.isSelected() ) printKey = true; else printKey = false;
      if ( mouse.isSelected() ) printMouse = true; else printMouse = false;
      if ( motion.isSelected() ) printMotion = true; else printMotion = false;
   }//itemStateChanged
}//CheckBoxJPanel


class Listeners implements ActionListener, ItemListener, WindowListener,
                                  ComponentListener, FocusListener, TextListener,
                                  KeyListener, MouseListener, MouseMotionListener,
                                  AdjustmentListener
{
   private CheckBoxJPanel ecbp;  // the event check box panel
   private CheckBoxJPanel ccbp;  // the component check box panel
   public static OutputJFrame output;

   Listeners(CheckBoxJPanel ecbp, CheckBoxJPanel ccbp)
   {
      this.ecbp = ecbp;  // the event check box panel
      this.ccbp = ccbp;  // the component check box panel
   }//constructor

   // implement the ActionListener interface
   public void actionPerformed(ActionEvent e)
   {  if (ecbp.printAction || ccbp.printAction) print(e); }

   // implement the ItemListener interface
   public void itemStateChanged(ItemEvent e)
   {  if (ecbp.printItem || ccbp.printItem) print(e); }

   // implement the WindowListener interface (seven methods)
   public void windowClosed(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowDeiconified(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowIconified(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowActivated(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowDeactivated(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowOpened(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }
   public void windowClosing(WindowEvent e)
   {  if (ecbp.printWindow || ccbp.printWindow) print(e); }

   // implement the ComponentListener interface (four methods)
   public void componentMoved(ComponentEvent e)
   {  if (ecbp.printComponent || ccbp.printComponent) print(e); }
   public void componentHidden(ComponentEvent e)
   {  if (ecbp.printComponent || ccbp.printComponent) print(e); }
   public void componentResized(ComponentEvent e)
   {  if (ecbp.printComponent || ccbp.printComponent) print(e); }
   public void componentShown(ComponentEvent e)
   {  if (ecbp.printComponent || ccbp.printComponent) print(e); }

   // implement the FocusListener interface
   public void focusGained(FocusEvent e)
   {  if (ecbp.printFocus || ccbp.printFocus) print(e); }
   public void focusLost(FocusEvent e)
   {  if (ecbp.printFocus || ccbp.printFocus) print(e); }

   // implement the TextListener interface
   public void textValueChanged(TextEvent e)
   {  if (ecbp.printText || ccbp.printText) print(e); }

   // implement the KeyListener interface (three methods)
   public void keyPressed(KeyEvent e)
   {  if (ecbp.printKey || ccbp.printKey) print(e); }
   public void keyReleased(KeyEvent e)
   {  if (ecbp.printKey || ccbp.printKey) print(e); }
   public void keyTyped(KeyEvent e)
   {  if (ecbp.printKey || ccbp.printKey) print(e); }

   // implement the MouseListener interface (five methods)
   public void mouseClicked(MouseEvent e)
   {  if (ecbp.printMouse || ccbp.printMouse) print(e); }
   public void mousePressed(MouseEvent e)
   {  if (ecbp.printMouse || ccbp.printMouse) print(e); }
   public void mouseReleased(MouseEvent e)
   {  if (ecbp.printMouse || ccbp.printMouse) print(e); }
   public void mouseEntered(MouseEvent e)
   {  if (ecbp.printMouse || ccbp.printMouse) print(e); }
   public void mouseExited(MouseEvent e)
   {  if (ecbp.printMouse || ccbp.printMouse) print(e); }

   // implement the MouseMotionListener interface
   public void mouseDragged(MouseEvent e)
   {  if (ecbp.printMotion || ccbp.printMotion) print(e); }
   public void mouseMoved(MouseEvent e)
   {  if (ecbp.printMotion || ccbp.printMotion) print(e); }

   // implement the the AdjustmentListener interface
   public void adjustmentValueChanged(AdjustmentEvent e)
   {  if (ecbp.printAdjustment || ccbp.printAdjustment) print(e);}

   // method for printing out an event object
   private void print(AWTEvent e)
   {
      output.out.append( output.extraline );
      output.out.append( e + "\n" );
   }//print
}//Listeners
